//
//  MyDocument.m
//  LilyRunner
//
//  Created by Toshi Nagata on 09/02/07.
//  Copyright Toshi Nagata 2009 . All rights reserved.
//
/*
 Copyright (c) 2009 Toshi Nagata. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "MyDocument.h"
#import "MyAppController.h"

@implementation MyDocument

#pragma mark ====== Console Output ======

- (int)appendMessage: (NSString *)string
{
	NSRange range;
	range.location = [[textView textStorage] length];
	range.length = 0;
	[textView replaceCharactersInRange: range withString: string];
	range.location = [[textView textStorage] length];
	[textView scrollRangeToVisible: range];
	return [string length];
}

- (void)setTypingColor: (NSColor *)color
{
	NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithDictionary: [textView typingAttributes]];
	[attr setObject: color forKey: NSForegroundColorAttributeName];
	[textView setTypingAttributes: attr];
}

- (void)flushTaskOutput: (NSTask *)aTask
{
	NSData *data;
	NSFileHandle *hd = [[aTask standardOutput] fileHandleForReading];
	data = [hd availableData];
	if (data != nil) {
		int len = [data length];
		char *buf = (char *)malloc(len + 1);
		if (buf != NULL) {
			memmove(buf, [data bytes], len);
			buf[len] = 0;
			[self appendMessage: [NSString stringWithUTF8String: buf]];
			free(buf);
		}
	}
}

#pragma mark ====== Invoke Lilypond ======

- (void)taskTerminated: (NSNotification *)aNotification
{	
	int status;
	if (task != [aNotification object])
		return;
	if (timer != nil) {
		[timer invalidate];
		timer = nil;
	}
	status = [task terminationStatus];
	if (status == 0) {
		NSString *pdfpath, *previewpath;
		NSString *path = [self fileName];
		[self flushTaskOutput: task];
		[self setTypingColor: [NSColor blueColor]];
		[self appendMessage: [NSString stringWithFormat: @"=== %@ terminated gracefully at %@\n", taskname, [[NSDate date] description]]];
		[self setTypingColor: [NSColor blackColor]];

		/*  Follow-up  */
		if ([taskname isEqualToString: @"lilypond"]) {
			pdfpath = [[path stringByDeletingPathExtension] stringByAppendingPathExtension: (productext != nil ? productext : @"pdf")];
			productext = nil;
			previewpath = [[NSUserDefaults standardUserDefaults] valueForKey: gPreviewPathKey];
			[[NSWorkspace sharedWorkspace] openFile: pdfpath withApplication: previewpath];
		} else if ([taskname isEqualToString: @"convert-ly"]) {
			/*  The converted file is XXX.ly~, and the backup file is XXX.ly~~  */
			/*  Rename as they should be  */
			NSFileManager *manager = [NSFileManager defaultManager];
			NSString *backpath = [path stringByAppendingString: @"~"];
			[manager removeFileAtPath: path handler: nil];
			[manager movePath: backpath toPath: path handler: nil];
			[manager movePath: [backpath stringByAppendingString: @"~"] toPath: backpath handler: nil];
		}
	} else {
		[self setTypingColor: [NSColor redColor]];
		[self appendMessage: [NSString stringWithFormat: @"*** %@ terminated with exit status %d at %@\n", taskname, status, [[NSDate date] description]]];
		[self setTypingColor: [NSColor blackColor]];
	}		
	[task autorelease];
	task = nil;
	[taskname release];
	taskname = nil;
}

- (void)doTimer: (NSTimer *)theTimer
{
	if (task != nil)
		[self flushTaskOutput: task];
}

- (void)invokeTask: (NSString *)tpath withName: (NSString *)tname directory: (NSString *)dirname arguments: (NSArray *)args
{
	NSString *str;
	NSPipe *pipe;
	
	[self setTypingColor: [NSColor blueColor]];
	
	/*  If task is already running for this document, stop it  */
	if (timer != nil) {
		[timer invalidate];
		timer = nil;
	}
	if (task != nil) {
		[task terminate];  /*  The interrupt message will be displayed from the notification handler */
		task = nil;
	}
	
	/*  Create a new NSTask instance  */
	task = [[NSTask alloc] init];
	
	/*  Set up redirection  */
	pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];
    [task setStandardError: [task standardOutput]];
	
	/*  The program to execute  */
	[task setLaunchPath: tpath];
	taskname = [tname retain];

	/*  Arguments  */
	[task setArguments: args];
	
	/*  The current directory  */
	[task setCurrentDirectoryPath: dirname];
	
	str = [NSString stringWithFormat: @"=== Invoking %@ at %@\n", taskname, [[NSDate date] description]];
	[self appendMessage: str];
	
	/*  Launch lilypond  */
    [task launch];
	
	/*  Set up timer for displaying messages  */
	timer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(doTimer:) userInfo: nil repeats: YES];
	
	[self setTypingColor: [NSColor blackColor]];
}

#pragma mark ====== Menu Commands ======

- (IBAction)compile: (id)sender
{
	NSString *lilypath, *path, *dname, *fname, *bname;
	NSArray *args;
	BOOL isDir;
	
	/*  The lilypond executable  */
	lilypath = [[NSUserDefaults standardUserDefaults] valueForKey: gLilyPondPathKey];
	if ([[NSFileManager defaultManager] fileExistsAtPath: lilypath isDirectory: &isDir] && isDir)
		lilypath = [lilypath stringByAppendingString: @"/Contents/Resources/bin/lilypond"];
	
	path = [self fileName];
	dname = [path stringByDeletingLastPathComponent];
	fname = [path lastPathComponent];
	bname = [[path lastPathComponent] stringByDeletingPathExtension];
	args = [NSArray arrayWithObjects: @"-o", bname, @"-fpdf", fname, nil];

	productext = @"pdf";
	
	[self invokeTask: lilypath withName: @"lilypond" directory: dname arguments: args];
}

- (IBAction)createPNG: (id)sender
{
	NSString *lilypath, *path, *dname, *fname, *bname, *res;
	NSArray *args;
	BOOL isDir;
	
	/*  The lilypond executable  */
	lilypath = [[NSUserDefaults standardUserDefaults] valueForKey: gLilyPondPathKey];
	if ([[NSFileManager defaultManager] fileExistsAtPath: lilypath isDirectory: &isDir] && isDir)
		lilypath = [lilypath stringByAppendingString: @"/Contents/Resources/bin/lilypond"];
	
	path = [self fileName];
	dname = [path stringByDeletingLastPathComponent];
	fname = [path lastPathComponent];
	bname = [[path lastPathComponent] stringByDeletingPathExtension];
	res = [NSString stringWithFormat: @"-dresolution=%@", [[NSUserDefaults standardUserDefaults] valueForKey: gPNGResolutionKey]];

	args = [NSArray arrayWithObjects: @"-o", bname, @"-fpng", res, fname, nil];
	
	productext = @"png";

	[self invokeTask: lilypath withName: @"lilypond" directory: dname arguments: args];
}

- (IBAction)updateSyntax: (id)sender
{
	NSString *path, *cvpath, *dname, *fbname;
	NSArray *args;
	NSFileManager *manager = [NSFileManager defaultManager];
	BOOL isDir;
	
	/*  The convert-ly executable  */
	cvpath = [[NSUserDefaults standardUserDefaults] valueForKey: gLilyPondPathKey];
	if ([[NSFileManager defaultManager] fileExistsAtPath: cvpath isDirectory: &isDir] && isDir)
		cvpath = [cvpath stringByAppendingString: @"/Contents/Resources/bin/convert-ly"];
	else {
		cvpath = [[cvpath stringByDeletingLastPathComponent] stringByAppendingPathComponent:@"convert-ly"];
	}
	
	/*  Copy the current file to a backup file  */
	/*  (convert-ly automatically creates a backup file, but this behavior interferes
	 with the NSDocument filename and leads to a confusion)  */
	path = [self fileName];
	fbname = [path stringByAppendingString: @"~"];
	if ([manager fileExistsAtPath: fbname])
		[manager removeFileAtPath: fbname handler: nil];
	[manager copyPath: path toPath: fbname handler: nil];
	dname = [fbname stringByDeletingLastPathComponent];
	
	args = [NSArray arrayWithObjects: @"-e", [fbname lastPathComponent], nil];
	
	[self invokeTask: cvpath withName: @"convert-ly" directory: dname arguments: args];
}

- (IBAction)openWithExternalEditor: (id)sender
{
	[[NSApp delegate] openEditorWithFile: [self fileName] line: 1 column: 1];
}

- (IBAction)clearMessage: (id)sender
{
	[textView setString: @""];
}

#pragma mark ====== NSDocument Overrides ======

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	if (timer != nil)
		[timer invalidate];
	if (task != nil) {
		[task terminate];
		[task release];
	}
	[super dealloc];
}

- (NSString *)windowNibName
{
	return @"MyDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)windowController
{
	NSFont *font;
	[super windowControllerDidLoadNib: windowController];

	font = [NSFont userFixedPitchFontOfSize: 12.0];
	[textView setFont: font];
	
	//  Register notification
	[[NSNotificationCenter defaultCenter]
	 addObserver:self
	 selector:@selector(taskTerminated:)
	 name:NSTaskDidTerminateNotification 
	 object:nil];
}

- (BOOL)readFromURL:(NSURL *)url ofType:(NSString *)typeName error:(NSError **)outError
{
	NSString *path;
	BOOL isDir;
	if ([url isFileURL] && (path = [url path]) != nil && [[NSFileManager defaultManager] fileExistsAtPath: path isDirectory: &isDir] && !isDir) {
		//  Schedule lilypond for start
		[[NSRunLoop currentRunLoop] performSelector: @selector(invokeLilypondWithPath:) target: self argument: path order: 0 modes: [NSArray arrayWithObject: NSDefaultRunLoopMode]];
		return YES;
	} else {
		if (outError != NULL)
			*outError = [NSError errorWithDomain: NSOSStatusErrorDomain code: fnfErr userInfo: NULL];
		return NO;
	}
}

@end
