//
//  PreferenceWindowController.m
//  LilyRunner
//
//  Created by Toshi Nagata on 09/02/08.
//  Copyright 2009 Toshi Nagata. All rights reserved.
//
/*
 Copyright (c) 2009 Toshi Nagata. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "PreferenceWindowController.h"
#import "MyAppController.h"

@implementation PreferenceWindowController

- (id)init
{
    self = [super initWithWindowNibName:@"PreferenceWindow"];
	if (self) {
	}
    return self;	
}

- (void)updateControls
{
	NSUserDefaults *def = [NSUserDefaults standardUserDefaults];
	[lilyPathText setStringValue: [def valueForKey: gLilyPondPathKey]];
	[previewPathText setStringValue: [def valueForKey: gPreviewPathKey]];
	[editorPathText setStringValue: [def valueForKey: gEditorPathKey]];
	[scriptView setString: [def valueForKey: gEditCommandKey]];
	[pngResolutionText setStringValue: [def valueForKey: gPNGResolutionKey]];
}

- (void)windowDidLoad
{
	[self updateControls];
}

- (IBAction)selectLily: (id)sender
{
	[[NSApp delegate] selectLily];
	[self updateControls];
}

- (IBAction)selectPreview: (id)sender
{
	[[NSApp delegate] selectPreview];
	[self updateControls];
}

- (IBAction)selectEditor: (id)sender
{
	[[NSApp delegate] selectEditor];
	[self updateControls];
}

- (IBAction)enterPNGResolution: (id)sender
{
	[[NSApp delegate] enterPNGResolution: sender];
	[self updateControls];
}

- (IBAction)selectScriptTemplate: (id)sender
{
	NSOpenPanel *panel;
	NSString *str, *val;
	int retval;
	panel = [NSOpenPanel openPanel];
	str = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: @"EditorScripts"];
	[panel setDirectory: str];
	[panel setAllowsMultipleSelection: NO];
	[panel setMessage: @"Select script template:"];
	retval = [panel runModal];
	[panel close];
	if (retval == NSFileHandlingPanelOKButton) {
		NSStringEncoding enc;
		NSError *err;
		val = [panel filename];
		str = [NSString stringWithContentsOfFile: val usedEncoding: &enc error: &err];
		if (str == nil) {
			[[NSAlert alertWithError: err] runModal];
			return;
		}
		[[NSUserDefaults standardUserDefaults] setValue: str forKey: gEditCommandKey];
		[self updateControls];
	}
}

- (void)endEditingScript
{
	NSUserDefaults *def = [NSUserDefaults standardUserDefaults];
	[def setValue: [scriptView string] forKey: gEditCommandKey];
	[def synchronize];
	[self updateControls];
}

- (void)windowDidResignKey:(NSNotification *)notification
{
	[self endEditingScript];
}

- (void)windowWillClose:(NSNotification *)notification
{
	[self endEditingScript];
}

- (void)textDidEndEditing:(NSNotification *)aNotification
{
	[self endEditingScript];
}

@end
